"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.aReduceRight = void 0;
/**
 * Async ReduceRight function
 *
 * @export
 * @template T
 * @template R
 * @param {T[]} elements
 * @param {CallBackReduceRight<T, R>} cb
 * @param {R} [initialValue]
 * @returns {Promise<T | R>}
 */
function aReduceRight(elements, cb, initialValue) {
    return __awaiter(this, void 0, void 0, function* () {
        if (!elements.length && initialValue === undefined) {
            throw new Error('Reduce of empty array with no initial value');
        }
        let reducedValue;
        let index = elements.length - 1;
        if (initialValue === undefined) {
            reducedValue = elements[index];
            index--;
        }
        else {
            reducedValue = initialValue;
        }
        for (; index >= 0; index--) {
            reducedValue = yield cb(reducedValue, elements[index], index, elements);
        }
        return reducedValue;
    });
}
exports.aReduceRight = aReduceRight;
//# sourceMappingURL=a_reduce_right.js.map